/*
	AVIInfo plugin for AviSynth -- The filter gives info about the streams in the clip.

	Copyright (C) 2005 Trevlac and Wilbert Dijkhof
  
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.
	 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
		
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	
	The author can be contacted at:
	wilbertdijkhof at yahoo.com
*/

#include "stdafx.h"
#include "avisynth.h"
#include <vfw.h>


class AVIInfo
{
public:
  char buf[50];

  AVIInfo::AVIInfo(const char filename[], const char* _info, int _stream, IScriptEnvironment* env);  
  AVIInfo::~AVIInfo();
private:
	int info;
	int stream;
};



// file name comes in
// the env thing is a reference back to avs so you can tell it things like errors.
AVIInfo::AVIInfo(const char filename[], const char* _info, int _stream, IScriptEnvironment* env) : stream(_stream)
{
  if (lstrcmpi(_info, "type") == 0)
    info = 0;
  else if (lstrcmpi(_info, "fourCC") == 0)
    info = 1;
  else if (lstrcmpi(_info, "framerate") == 0)
    info = 2;
  else if (lstrcmpi(_info, "width") == 0)
    info = 3;
  else if (lstrcmpi(_info, "height") == 0)
    info = 4;
  else if (lstrcmpi(_info, "length") == 0)
    info = 5;
  else if (lstrcmpi(_info, "skew") == 0)
    info = 6;
  else
    env->ThrowError("AVIInfo: info must be \"type\", \"fourCC\", \"framerate\", \"width\", \"height\", \"length\" or \"skew\"");

// The variables I need
// Most are structures to hold the AVI file info data

// Return code from file open.  0 is good
  LONG hr;

// Reference to file after you open it
  PAVIFILE pfile;

// Structure to hold File Info
  AVIFILEINFO pfi;

// Reference to a stream in the file
  PAVISTREAM ppavi;

// Structure to hold Stream Info (where the fccHandler is)
  AVISTREAMINFO psi;

// holds a return code from getting the Stream Info
  HRESULT hresult;
    
  AVIFileInit(); // opens AVIFile library

// Open the file. If you don't get a zero back, Tell avs there was an error
  hr = AVIFileOpen(&pfile, filename, OF_READ, NULL);
  if (hr != 0){
    env->ThrowError("AVIInfo autodetect: couldn't open file\nError code: %d", GetLastError());
    AVIFileExit();
	return;
  }
 
// Get the File Info into our local structure so we can look
  AVIFileInfo(pfile, &pfi, sizeof(pfi));

// Check that stream exists
  if ((stream > pfi.dwStreams) || (stream < 1))
    env->ThrowError("AVIInfo: stream doesn't exist; lower the value of \"stream\"");

// Open the stream
  AVIFileGetStream(pfile, &ppavi, NULL, stream-1);
 
  // If there is a stream ... Get its Info so we can look
  if (ppavi != NULL) {
    hresult = AVIStreamInfo(ppavi, &psi, sizeof(psi));

    if (info==0) { // type of stream
      char vs[] = "videostream", as[] = "audiostream";
      if (psi.fccType == streamtypeVIDEO) { 
        sprintf(buf,"%s",vs);
	  } else if (psi.fccType == streamtypeAUDIO) {
      sprintf(buf,"%s",as);
	  } else {
        env->ThrowError("AVIInfo: stream doesn't exist");
	  }
	}
	else if (info==1) { // fourCC
      // Translate the bits of the fccHandler into a string buf which is global to our class
      sprintf(buf,"%c%c%c%c",psi.fccHandler, psi.fccHandler>>8, psi.fccHandler>>16, psi.fccHandler>>24); }
	else if (info==2) { // fps
      // dwRate video - 1000000 (always ?)
      // dwRate audio - byter per second
      // dwScale video - duration of one frame [mus]
      // dwScale audio - no. of bytes per sample
      float fps = float(psi.dwRate)/psi.dwScale; // framerate (audio: samplerate)
      sprintf(buf,"%f",fps);
	}
	else if (info==3) { // width
      sprintf(buf,"%i",pfi.dwWidth); // width of stream
	}
	else if (info==4) { // height
      sprintf(buf,"%i",pfi.dwHeight); // height of stream
	}
	else if (info==5) { // no. of frames (no. of samples)
      sprintf(buf,"%i",psi.dwLength); // no. of frames (no. of samples)
	}
	else if (info==6) { // no. of frames (no. of samples)
      sprintf(buf,"%i",psi.dwInitialFrames); // audio skew
      // sprintf(buf,"%i",psi.nChannels);
	  // http://msdn.microsoft.com/library/default.asp?url=/library/en-us/multimed/htm/_win32_waveoutopen.asp
	}
  }

  // Clean up
  if(pfile) AVIFileRelease(pfile); // closes the file
    AVIFileExit(); // releases AVIFile library
}

// info about avi-streams:
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/multimed/htm/_win32_reading_streams_from_an_avi_file.asp

AVSValue __cdecl Create_AVIInfo(AVSValue args, void* user_data, IScriptEnvironment* env)
{
  return (new AVIInfo(args[0].AsString(), args[1].AsString(0), args[2].AsInt(1), env))->buf;
}



extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
  env->AddFunction("AVIInfo", "s[info]s[stream]i", Create_AVIInfo, 0);
  return "`AVIInfo' AVIInfo plugin";
}